@@ -2,41 +2,68 @@ use std::ops::{Add, Sub};
 
 use chrono::{Duration, NaiveTime, Timelike};
 
-use super::{
-    category::Category,
-    event::{EventInstance, EventSchedule, EventScheduleIter},
-};
+use crate::schedule::EventSchedule;
+
+use super::{category::Category, event::EventInstance};
 
 pub struct Meta {
     pub name: String,
     pub category: Category,
-    pub events: Vec<EventSchedule>,
+    pub schedules: Vec<EventSchedule>,
 }
 
-pub struct MetaIter {
+pub struct IntoIter {
     /// Respresents the amount of time from UTC 00:00
     current_time: Duration,
-    schedule: Vec<EventSchedule>,
+
+    /// A meta's schedule is a collection of Event's (in that meta) schedules
+    schedules: Vec<EventSchedule>,
 }
 
-impl MetaIter {
-    pub fn new(meta_key: MetaKey, current_time: NaiveTime) -> Self {
+impl IntoIter {
+    pub fn new<'a>(meta_key: &'a MetaKey, current_time: NaiveTime) -> Self {
         Self {
             current_time: Duration::seconds(current_time.num_seconds_from_midnight() as i64),
-            schedule: meta_key.info().events,
+            schedules: meta_key.info().schedules,
         }
     }
+
+    /// Skip to a certain time of day
+    pub fn time(mut self, time: NaiveTime) -> Self {
+        self.current_time = Duration::seconds(time.num_seconds_from_midnight() as i64);
+        self
+    }
+
+    /// Skip forward an amount of time
+    pub fn fast_foward(mut self, amount: Duration) -> Self {
+        self.current_time = self.current_time.add(amount);
+        self
+    }
+
+    pub fn now(&self) -> Option<EventInstance> {
+        self.schedules
+            .iter()
+            .filter_map(|event_schedules| {
+                event_schedules
+                    .into_iter()
+                    .fast_foward(self.current_time)
+                    .now()
+            })
+            .next()
+    }
 }
 
-impl Iterator for MetaIter {
+impl Iterator for IntoIter {
     type Item = EventInstance;
 
     fn next(&mut self) -> Option<EventInstance> {
-        let next_event = self
-            .schedule
+        let next_event: EventInstance = self
+            .schedules
             .iter()
-            .map(|event| {
-                EventScheduleIter::new(event, self.current_time)
+            .map(|event_schedule| {
+                event_schedule
+                    .into_iter()
+                    .fast_foward(self.current_time)
                     .next()
                     .unwrap()
             })
@@ -51,11 +78,21 @@ impl Iterator for MetaIter {
                 }
             })
             .unwrap();
-        self.current_time = next_event.start_time + next_event.schedule.length;
+        self.current_time = next_event.start_time;
         Some(next_event)
     }
 }
 
+impl IntoIterator for MetaKey {
+    type Item = EventInstance;
+
+    type IntoIter = IntoIter;
+
+    fn into_iter(self) -> Self::IntoIter {
+        IntoIter::new(&self, NaiveTime::from_hms(0, 0, 0))
+    }
+}
+
 pub enum MetaKey {
     DayAndNight,
     WorldBosses,
@@ -114,7 +151,7 @@ impl MetaKey {
             MetaKey::DayAndNight => Meta {
                 name: "Day and Night".to_string(),
                 category: Category::CoreTyria,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "Dawn".to_string(),
                         offset: NaiveTime::from_hms(0, 25, 0),
@@ -144,7 +181,7 @@ impl MetaKey {
             MetaKey::WorldBosses => Meta {
                 name: "World Bosses".to_string(),
                 category: Category::CoreTyria,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "Admiral Taidha Covington".to_string(),
                         offset: NaiveTime::from_hms(0, 0, 0),
@@ -210,7 +247,7 @@ impl MetaKey {
             MetaKey::HardWorldBosses => Meta {
                 name: "Hard World Bosses".to_string(),
                 category: Category::CoreTyria,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "Tequatl the Sunless".to_string(),
                         offset: NaiveTime::from_hms(0, 0, 0),
@@ -324,7 +361,7 @@ impl MetaKey {
             MetaKey::LeyLineAnomaly => Meta {
                 name: "Ley-Line Anomaly".to_string(),
                 category: Category::CoreTyria,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "Timberline Falls".to_string(),
                         offset: NaiveTime::from_hms(0, 20, 0),
@@ -348,7 +385,7 @@ impl MetaKey {
             MetaKey::PVPTournaments => Meta {
                 name: "PvP Tournaments".to_string(),
                 category: Category::CoreTyria,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "Balthazar's Brawl".to_string(),
                         offset: NaiveTime::from_hms(0, 0, 0),
@@ -378,7 +415,7 @@ impl MetaKey {
             MetaKey::DryTop => Meta {
                 name: "Dry Top".to_string(),
                 category: Category::LivingWorldSeason2,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "Crash Site".to_string(),
                         offset: NaiveTime::from_hms(0, 0, 0),
@@ -396,7 +433,7 @@ impl MetaKey {
             MetaKey::VerdantBrink => Meta {
                 name: "Verdant Brink".to_string(),
                 category: Category::HeartOfThorns,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "Night: Night and the Enemy".to_string(),
                         offset: NaiveTime::from_hms(1, 45, 0),
@@ -420,7 +457,7 @@ impl MetaKey {
             MetaKey::AuricBasin => Meta {
                 name: "Auric Basin".to_string(),
                 category: Category::HeartOfThorns,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "Challenges".to_string(),
                         offset: NaiveTime::from_hms(0, 45, 0),
@@ -450,7 +487,7 @@ impl MetaKey {
             MetaKey::TangledDepths => Meta {
                 name: "Tangled Depths".to_string(),
                 category: Category::HeartOfThorns,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "Prep".to_string(),
                         offset: NaiveTime::from_hms(0, 25, 0),
@@ -474,7 +511,7 @@ impl MetaKey {
             MetaKey::DragonsStand => Meta {
                 name: "Dragon's Stand".to_string(),
                 category: Category::HeartOfThorns,
-                events: vec![EventSchedule {
+                schedules: vec![EventSchedule {
                     name: "Start advancing on the Blighting Towers".to_string(),
                     offset: NaiveTime::from_hms(1, 30, 0),
                     frequency: Duration::hours(2),
@@ -484,7 +521,7 @@ impl MetaKey {
             MetaKey::LakeDoric => Meta {
                 name: "Lake Doric".to_string(),
                 category: Category::LivingWorldSeason3,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "Noran's Homestead".to_string(),
                         offset: NaiveTime::from_hms(0, 30, 0),
@@ -508,7 +545,7 @@ impl MetaKey {
             MetaKey::CrystalOasis => Meta {
                 name: "Crystal Oasis".to_string(),
                 category: Category::PathOfFire,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "Rounds 1 to 3".to_string(),
                         offset: NaiveTime::from_hms(0, 5, 0),
@@ -526,7 +563,7 @@ impl MetaKey {
             MetaKey::DesertHighlands => Meta {
                 name: "Desert Highlands".to_string(),
                 category: Category::PathOfFire,
-                events: vec![EventSchedule {
+                schedules: vec![EventSchedule {
                     name: "Buried Treasure".to_string(),
                     offset: NaiveTime::from_hms(1, 0, 0),
                     frequency: Duration::hours(2),
@@ -536,7 +573,7 @@ impl MetaKey {
             MetaKey::ElonRiverlands => Meta {
                 name: "Elon Riverlands".to_string(),
                 category: Category::PathOfFire,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "The Path to Ascension: Augury Rock".to_string(),
                         offset: NaiveTime::from_hms(1, 30, 0),
@@ -554,7 +591,7 @@ impl MetaKey {
             MetaKey::TheDesolation => Meta {
                 name: "The Desolation".to_string(),
                 category: Category::PathOfFire,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "Junudu Rising".to_string(),
                         offset: NaiveTime::from_hms(0, 30, 0),
@@ -578,7 +615,7 @@ impl MetaKey {
             MetaKey::DomainOfVabbi => Meta {
                 name: "Domain of Vabbi".to_string(),
                 category: Category::PathOfFire,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "Forged with Fire".to_string(),
                         offset: NaiveTime::from_hms(0, 0, 0),
@@ -596,7 +633,7 @@ impl MetaKey {
             MetaKey::DomainOfIstan => Meta {
                 name: "Domain of Istan".to_string(),
                 category: Category::LivingWorldSeason4,
-                events: vec![EventSchedule {
+                schedules: vec![EventSchedule {
                     name: "Palawadan".to_string(),
                     offset: NaiveTime::from_hms(1, 45, 0),
                     frequency: Duration::hours(2),
@@ -606,7 +643,7 @@ impl MetaKey {
             MetaKey::JahaiBluffs => Meta {
                 name: "Jahai Bluffs".to_string(),
                 category: Category::LivingWorldSeason4,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "Escorts".to_string(),
                         offset: NaiveTime::from_hms(1, 0, 0),
@@ -624,7 +661,7 @@ impl MetaKey {
             MetaKey::ThunderheadPeaks => Meta {
                 name: "Thunderhead Peaks".to_string(),
                 category: Category::LivingWorldSeason4,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "The Oil Floes".to_string(),
                         offset: NaiveTime::from_hms(0, 45, 0),
@@ -642,7 +679,7 @@ impl MetaKey {
             MetaKey::GrothmarValley => Meta {
                 name: "Grothmar Valley".to_string(),
                 category: Category::TheIcebroodSaga,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "Effigy".to_string(),
                         offset: NaiveTime::from_hms(0, 10, 0),
@@ -672,7 +709,7 @@ impl MetaKey {
             MetaKey::BjoraMarches => Meta {
                 name: "Bjora Marches".to_string(),
                 category: Category::TheIcebroodSaga,
-                events: vec![
+                schedules: vec![
                     EventSchedule {
                         name: "Shards and Construct".to_string(),
                         offset: NaiveTime::from_hms(0, 0, 0),
@@ -702,7 +739,7 @@ impl MetaKey {
             MetaKey::Dragonstorm => Meta {
                 name: "Dragonstorm".to_string(),
                 category: Category::TheIcebroodSaga,
-                events: vec![EventSchedule {
+                schedules: vec![EventSchedule {
                     name: "Dragonstorm (Public)".to_string(),
                     offset: NaiveTime::from_hms(1, 0, 0),
                     frequency: Duration::hours(2),
